#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 22-Apr-2006
################################################################################
#
# Private - Script part of Ethereal Realms, designed to send private messages
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Comm;						# Communication handler
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler
use Ethereal::Who;						# Who is on handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $comm;							# Communications panel
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $tmpl;							# Template file

my $count;							# Amount of targets
my $post;							# Temporary post
my $target;							# Individual targets
my @targets;							# Targets to send to

my %realm;							# Realm hash


################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);


	# Fetch target
	@targets = $cgi->param('TARGET');

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Template instance
	$tmpl = new Ethereal::Template(\%realm);


	# Login and Comm handles
	$login = new Ethereal::Login($database, $cgi, $param);
	$comm  = new Ethereal::Comm($database, $cgi, $tmpl);

	# Authenthication
	if ($login->GetVerificationRealm())
	{
		# Prevent bans
		$login->CheckLegality(\%realm);

		# Only do the following if posting is needed
		if ((defined($param->{'PSEND'}))
		 && (defined($param->{'TARGET'})))
		{
			# Only if target is valid
			if ($targets[0] ne $realm{'TxtNone'})
			{
				# Retreive count
				$count = @targets;

				# Determin post
				$post = ($count > 1) ? "$realm{TagMulti} $param->{POST}" : "$param->{POST}";

				# Circle through hosts
				foreach $target (@targets)
				{
					# Set value of target
					$param->{'TARGET'} = $target;
					$param->{'POST'}   = $post;
		
					TargetHandler($database, $param, \%realm);
					$comm->DataWrite($param);
				}
			}
		}

		# Display panel
		DisplayHandler($database, $cgi, $param, $comm, \%realm);
	}


################################################################################
# Sub-Routines
################################################################################

####################
# Display Handler
#
# Display prompt for a private message, this includes a list of all the puppets
# et cetera. 

sub DisplayHandler
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Cgi handle
	my $param    = shift;					# Parameter handle
	my $comm     = shift;					# Communications panel
	my $realm    = shift;					# Realm options

	my $option;						# Option handle
	my $who;						# Who is on handle

	my $access;						# Access control
	my $admin;						# Admin priviliges
	my $js;							# Javascript control

	my $url;						# Self referencing link
	my $inline;						# Inline parameters

	my $defcolor;						# Default colour
	my $defface;						# Default face
	my $defsize;						# Default size
	my $multi;						# Multiple harassment charges
	my $panel;						# Private message panel

	my @who;						# List of puppets
	my @char;						# List of characters
	my @color;						# Colour choices
	my @face;						# Font face
	my @size;						# Text colour

	my %args;						# Default arguments
	my %who;						# Associative hash
	

	####################
	# Program area

	# Create instance
	$option   = new Ethereal::Option();
	$who      = new Ethereal::Who($database, $cgi, $param);


	# Pull values
	($access) = $database->DataGetRealmAccess($param->{'ROOM'});
	($admin)  = $database->DataGetAdmin($param->{'USER'});
	($js)     = $database->DataGetJavascript($param->{'USER'});


	# Populate arrays
        @color = $comm->FontSelectColor($param);
        @size  = $comm->FontSelectSize($param);
        @face  = $comm->FontSelectFace($param);

	# Populate puppet list
	$database->GroupList(\@char, $access, $param->{'USER'}, $param->{'ROOM'}); 

	# Who is on
	# Pull list
	if (defined($param->{'PESKY'}))
	{
		# Global menace to society
		$who->Pesky(\@who, \%who, $param->{'ROOM'});

		# Crazy to allow multiple
		$multi = 'false';
	}

	else
	{
		# Realm specific private
		$who->Private(\@who, \%who, $param->{'ROOM'});

		# El spam
		$multi = 'true';
	}


	# Sort colours
	@color = sort(@color);

	
	# Retreive size and color
	($defsize, $defcolor, $defface) = $database->DataGetFont($param->{'CHAR'}, $param->{'USER'}, $color[0], $face[0]);

	# Allow for system messages
	if ($admin eq 'yes')
	{
		# To array
		push(@who, $realm->{'TxtSystem'}); 

		# To associative hash
		$who{$realm->{'TxtSystem'}} = $realm->{'TxtSystem'};
	}

	# Default selection
	# Array
	unshift(@who, $realm->{'TxtNone'});

	# Hash
	$who{$realm->{'TxtNone'}} = $realm->{'TxtNone'};


	# Targets to who
	$target = (defined($param->{'OSELECT'})) ? $param->{'OSELECT'} : $who[0];

	# Cleanup for embed
	$param->Cleanup('POST');


	# Retreive private message panel
	$panel = $database->FetchRealmPrivate($param->{'ROOM'});


	# Javascript controls
	if ($js eq 'yes')
	{
		# Pull body tag
		if ($panel =~ /<(BODY.*?)>/si)
		{
			# Assign
			my $org = $1;
			my $new = $1;

			# Prepare
			$new =~ s/(\W)/\\$1/gs;
			

			# Replace
		 	$panel =~ s/$new/$org onload="document.SEND.POST.focus();"/gs;	
		}
	}


	# Default arguments
	%args = (
		USER  => $param->{'USER'},
		CRYPT => $param->{'CRYPT'},
		ROOM  => $param->{'ROOM'}, 
		CHAR  => $param->{'CHAR'}
	);

	# Pesky
	$args{'PESKY'} = 'True' unless (defined($param->{'PESKY'}));

	# Password protection
	$args{'RCRYPT'} = $param->{'RCRYPT'} if (defined($param->{'RCRYPT'}));
	
	# Self referencing links
	$url    = $cgi->url(-absolute=>1);
	$inline = $param->EmbedInline(%args);
		

	# Widgets
	# Assigning values
	my $wpostsend = $cgi->submit(
		-name     => $realm->{'TxtPost'},
		-value    => $realm->{'TxtPost'},
		-tabindex => 4);

	# Posting window
	my $wposttext = $cgi->textarea(
		-name     => 'POST',
		-rows     => 3,
		-columns  => 55,
		-wrap     => 'SOFT',
		-tabindex => 1);

	# Selection
	# Target handle
	my $wchartrgt = $cgi->scrolling_list(
		-name     => 'TARGET', 
		-values   => \@who,
		-labels   => \%who,
		-default  => $target,
		-size     => 5,
		-multiple => $multi,
		-tabindex => 3);

	# Our own
	my $wcharslct = (defined($database->{'DEFAULT'}))
	  ? $cgi->hidden('CHAR', $database->{'DEFAULT'})
	  : $cgi->popup_menu(
		-name     => 'CHAR',
		-values   => \@char,
		-default  => $param->{'CHAR'},
		-tabindex => 2);

	# Font handling
	# Font colour
	my $wfontcolr = $cgi->popup_menu(
		-name     => 'COLOR',
		-values   => \@color,
		-default  => $defcolor,
		-tabindex => 5);

	# Font face
	my $wfontface = $cgi->popup_menu(
		-name     => 'FACE',
		-values   => \@face,
		-default  => $defface,
		-tabindex => 6);

	# Font size
	my $wfontsize = $cgi->popup_menu(
		-name     => 'SIZE',
		-values   => \@size,
		-default  => $defsize,
		-tabindex => 7);


	# Form values
	my $wformstrt = $cgi->start_form(-name=>'SEND');
	my $wformhidn = $param->EmbedNormal($param->Flat(), PSEND=>'True');
	my $wformends = $cgi->end_form();


	# Link replacement
	$panel =~ s/LWPESKY/HREF=\"$url$inline\"/gs;

	# Widget replacements
	# Posting widgets
	$panel =~ s/WPOSTSEND/$wpostsend/gs;
	$panel =~ s/WPOSTTEXT/$wposttext/gs;

	# Puppet selection
	$panel =~ s/WCHARSLCT/$wcharslct/gs;
	$panel =~ s/WCHARTRGT/$wchartrgt/gs;

	# Font selection
	$panel =~ s/WFONTCOLR/$wfontcolr/gs;
	$panel =~ s/WFONTFACE/$wfontface/gs;
	$panel =~ s/WFONTSIZE/$wfontsize/gs;

	# Form
	$panel =~ s/WFORMSTRT/$wformstrt$wformhidn/gs;
	$panel =~ s/WFORMENDS/$wformends/gs;


	# Display HTML
	# Document
	print $panel, "\n";
}

#####################
# Target Handler
#
# Will determine target to send the message privately to.  This will also create a
# parameter called SOURCE which will receive a copy as well.

sub TargetHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Chat options

	my $pool;						# Puppeteer pool
	my $target;						# Target


	#####################
	# Program area

	# Only do changes if target exists
	if (defined($param->{'TARGET'}))
	{
		# For future selection
		$param->{'OSELECT'} = $param->{'TARGET'};


		# Only retreive if needed
		if (($param->{'TARGET'} ne 'dice roll') 
		 && ($param->{'TARGET'} ne $realm->{'TxtSystem'}))
		{ 
			# Split into approriate
			$pool   = substr($param->{'TARGET'}, 0, 4);
			$target = substr($param->{'TARGET'}, 4); 
			

			# Retreive
			# Determin query to run
			if ($pool eq 'MAIN')
			{	
				# Human readable
				$param->{'OTARGET'} = $target;

				# System level handles
				($target) = $database->DataGetPuppeteerLogin($target); 
			}
			else
			{
				# Human readable
				$param->{'OTARGET'} = $target . '*';

				# Source level handles
				($target) = $database->DataGetPuppeteerCanon($param->{'ROOM'}, $target); 
			}

			# Assign target
			$param->{'TARGET'} = $target;
		}

		else
		{
			# So system understands what to do
			$param->{'OTARGET'} = $param->{'TARGET'};
		}

		# Assign values to parameters
		$param->{'SOURCE'} = $param->{'USER'};
	}
}
