#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 17-Nov-2005
################################################################################
#
# Reset - Used to reset the password for a specific username
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Mail;						# Mail handling
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $send;							# Mail name
my $param;							# Parameter hash
my $tmpl;							# Template handling

my $ban;							# Banning
my $crypt;							# Encypted passwords
my $login;							# Login name
my $message;							# Error message

my %config;							# Configuration hash
my %system;							# System hash


#################################################################################
# Program Area
#################################################################################

	
	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Establish connection
	$database->Connect($cgi);

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);

	# Retreive parameters
	$param->GetParam();

	# Retreive hash
	$database->GetHashUser(\%config);


	# Create instances
	$tmpl  = new Ethereal::Template(\%config);


	# If no registration codes or logins		
	if (defined($param->{'LOGIN'}))
	{
		# Pull Email address
		my ($login, $ban, $adm, $sup) = $database->DataGet("SELECT
			 PuppeteerLogin,
			 PuppeteerBan,
			 PuppeteerAdminPrivs,
			 PuppeteerSuperPrivs
			FROM Puppeteer
			WHERE LOWER(PuppeteerLogin)=LOWER(?)",

		 $param->{'LOGIN'}
		);


		# Verified
		if ((defined($login))
		 && ($ban eq 'no')
		 && ($adm eq 'no')
		 && ($sup eq 'no'))
		{
			# Evaluate
			eval
			{
				# Get hash
				$database->GetHashSystem(\%system);


				# Pull information
				my ($email) = $database->DataGetPuppeteerEmail($login);
				my ($name)  = $database->DataGetPuppeteerName($login);

				# Password reset
				my ($reset) = $database->DocumentGetReset();


				# Generate new password
				$crypt = md5_hex(rand());
				$crypt = substr($crypt, 0, 8);


				# Mail sending
				# Initiate handle
				$send = tie(*MAIL, 'Ethereal::Mail', $database);

				# Initial setup
				$send->SetSubject("Your password for $login has been reset on $system{SetInfoServer}");
				$send->SetSearch(
					MMAIL => $email,
					MNAME => $name,
					MUSER => $login,
					MPASS => $crypt
				 );

				# Recipients
				$send->AddTo("$name <$email>");

				# Message
				print MAIL $reset;

				# Close and send 
				close(MAIL);
	

				# Update password
				$database->Write("UPDATE Puppeteer
					SET PuppeteerPassword=?
					WHERE PuppeteerLogin=?",

				  crypt($crypt, $crypt),
				  $login
				);

				# Commit
				$database->Commit();


				# Success
				$message = $tmpl->Pass('MsgRstPass', MMAIL=>$email); 
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();


				# Failure
				$message = $config{'MsgRstFail'};
			}
		}

		# Invalid access
		else
		{
			# Failure
			$message = $config{'MsgRstFail'};
		}
	}

	# Safety
	$message = (defined($message))
	  ? $message
	  : '';
		

	# HTML Display
	print $cgi->header();
	print $database->DocumentGetHeader(), "\n";
	  
	# Form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";


	# Display template
	$tmpl->Show('TmplReset',
		MMESS     => $message,
		WUSERNAME => $cgi->textfield('LOGIN', '', 10, 10),
		WRESET    => $cgi->submit($config{'TxtReset'})
	 );
		

	# End form
	print $cgi->end_form();

	# Footer
	print $database->DocumentGetFooter(), "\n";


################################################################################
# Sub-Routines
################################################################################
